package cn.rankin.productservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.PriceTypeEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.productservice.entity.ItemPrice;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

public interface ItemPriceRepository extends BasicJpaRepository<ItemPrice, String> {

    List<ItemPrice> findByItemId(String itemId);

    @Query(value = "select p from ItemPrice p where p.itemId = :itemId and p.type = :type and p.status = :status")
    List<ItemPrice> findByItemIdAndType(@Param("itemId") String itemId, @Param("type") PriceTypeEnum type, @Param("status") BaseStatusEnum status);

    @Query(value = "select p from ItemPrice p where p.itemId = :itemId and p.type = :type and p.status = :status and p.packageId = :packageId")
    List<ItemPrice> findByTypeAndPackageId(@Param("itemId") String itemId, @Param("type") PriceTypeEnum type, @Param("packageId") String packageId, @Param("status") BaseStatusEnum status);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "update ItemPrice p set p.status = :status where p.itemId = :itemId")
    Integer deleteByItemId(@Param("itemId") String itemId, @Param("status") BaseStatusEnum status);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "update ItemPrice p set p.status = :status where p.id not in (:ids)")
    Integer deleteExcludeId(@Param("ids") List ids, @Param("status") BaseStatusEnum status);

}
