package cn.rankin.productservice.repository;

import cn.rankin.common.utils.enums.ItemStatusEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.productservice.entity.Item;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import javax.transaction.Transactional;
import java.util.List;

public interface ItemRepository extends BasicJpaRepository<Item, String> {

    @Query(value = "select i from Item i where i.subId = :subId and i.status = :status")
    List<Item> findBySubId(@Param("subId") String subId, @Param("status") ItemStatusEnum status);

    List<Item> findAllBySubId(String subId);

    @Query(value = "select i from Item i where i.id in (:ids) and i.status <> :status")
    List<Item> findAllByIdIn(@Param("ids") List<String> ids, @Param("status") ItemStatusEnum status);

    @Transactional
    @Modifying(clearAutomatically = true)
    @Query(value = "update Item i set i.status = :status where i.id = :itemId")
    Integer updateStatusById(@Param("itemId") String itemId, @Param("status") ItemStatusEnum status);
}
