package cn.rankin.productservice.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.productservice.entity.ItemTagRelation;
import cn.rankin.productservice.repository.ItemTagRelationRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class ItemTagRelationService {

    @Autowired
    private ItemTagRelationRepository relationRepository;

    public List<ItemTagRelation> findAllByTagId(String id) {
        return relationRepository.findAllByTagId(id);
    }

    public List<ItemTagRelation> findByTagId(String id) {
        return relationRepository.findByTagId(id, BaseStatusEnum.NORMAL);
    }

    public List<ItemTagRelation> findAllByItemId(String itemId) {
        return relationRepository.findAllByItemId(itemId);
    }

    @Transactional
    public Integer deleteByItemId(String itemId) {
        return relationRepository.deleteByItemId(itemId, BaseStatusEnum.DEL);
    }

    @Transactional
    public Integer deleteByTagId(String tagId) {
        return relationRepository.deleteByTagId(tagId, BaseStatusEnum.DEL);
    }

    @Transactional
    public void updateByItem(String itemId, List<String> tagIdList) {
        if (tagIdList == null) {
            return;
        }else if (tagIdList.size() == 0) {
            deleteByItemId(itemId);
            return;
        }

        List<ItemTagRelation> relationList = relationRepository.findAllByItemId(itemId);

        List<String> currentIdList = new ArrayList<>();
        relationList.forEach( relation -> {
            String tagId = relation.getTagId();
            if (!tagIdList.contains(tagId)) {
                relation.setStatus(BaseStatusEnum.DEL);
            }else {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }
            currentIdList.add(tagId);
        });

        List<String> notExistTagIdList = ListUtil.subtract(tagIdList, currentIdList);
        if (notExistTagIdList != null && notExistTagIdList.size() > 0) {
            notExistTagIdList.forEach( tagId -> {
                ItemTagRelation relation = new ItemTagRelation();
                relation.setItemId(itemId);
                relation.setTagId(tagId);
                relation.setSort(0);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        relationRepository.save(relationList);
    }

    @Transactional
    public void updateByTag(String tagId, List<String> itemIdList) {
        if (itemIdList == null) {
            return;
        }else if (itemIdList.size() == 0) {
            deleteByItemId(tagId);
            return;
        }

        List<ItemTagRelation> relationList = relationRepository.findAllByTagId(tagId);

        List<String> currentIdList = new ArrayList<>();
        relationList.forEach( relation -> {
            String itemId = relation.getItemId();
            if (itemIdList.contains(itemId)) {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }else {
                relation.setStatus(BaseStatusEnum.DEL);
            }
            currentIdList.add(itemId);
        });

        List<String> notExistItemIdList = ListUtil.subtract(itemIdList, currentIdList);
        if (notExistItemIdList != null && notExistItemIdList.size() > 0) {
            notExistItemIdList.forEach( itemId -> {
                ItemTagRelation relation = new ItemTagRelation();
                relation.setItemId(itemId);
                relation.setTagId(tagId);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        // 排序
        relationList.forEach( relation -> {
            String itemId = relation.getItemId();
            Integer index = itemIdList.indexOf(itemId);
            relation.setSort(index);
        });

        relationRepository.save(relationList);
    }
}
