package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.product.WareDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.productservice.entity.Ware;
import cn.rankin.productservice.repository.WareRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class WareService {

    @Autowired
    private WareRepository wareRepository;

    public APIResult<Page<Ware>> search(Ware ware, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = wareRepository.count(ware);
        Page<Ware> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Ware> wareList = wareRepository.find(ware, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(wareList);

        return APIResult.ok(page);
    }

    public boolean exists(String code) {
        Long count = wareRepository.countByCode(code);
        return count > 0L;
    }

    @Transactional
    public APIResult<Ware> create(WareDTO wareDTO) {
        Ware ware = convert(wareDTO);
        if (exists(ware.getCode())) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }
        Ware result = wareRepository.save(ware);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Ware> update(WareDTO wareDTO) {
        Ware ware = convert(wareDTO);
        Ware result = wareRepository.update(ware);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = wareRepository.deleteAllById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    public APIResult<Ware> findById(String id) {
        Ware ware = wareRepository.find(id);
        if (ware == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(ware);
    }

    public APIResult<List<Ware>> findByIds(List<String> ids) {
        List<Ware> wareList = wareRepository.findByIdIn(ids, BaseStatusEnum.NORMAL);

        if (wareList == null || wareList.size() == 0) {
            return APIResult.ok(wareList);
        }

        Map<String, Ware> wareMap = ListUtil.convert(wareList, "id", Ware.class);
        List<Ware> sortWareList = new ArrayList<>();
        ids.forEach( id -> {
            Ware ware = wareMap.get(id);
            if (ware != null) {
                sortWareList.add(ware);
            }
        });

        return APIResult.ok(sortWareList);
    }
}
